@@ -1,4 +1,4 @@
-# (C) Copyright David Hastie and Silvia Liverani, 2012.
+# (C) Copyright David Hastie, Silvia Liverani and Aurore J. Lavigne, 2012-2014.
 
 # PReMiuM++ is free software; you can redistribute it and/or modify it under the
 # terms of the GNU Lesser General Public License as published by the Free Software
@@ -36,20 +36,36 @@ generateSampleDataFile<-function(clusterSummary){
 	nFixedEffects<-clusterSummary$nFixedEffects
 	nCategoriesY<-clusterSummary$nCategoriesY
 	if (is.null(nCategoriesY)) nCategoriesY<-1
+	includeCAR=clusterSummary$includeCAR
+	nClusters=length(subjectsPerCluster)
    
 	# Clustering covariates X
 	X<-matrix(NA,nSubjects,nCovariates)
 
 	k<-1
+
+	#sample k for spatial CAR model
+	if (includeCAR){
+		VectoSample=c()
+		for (k in 1:nClusters){
+			VectoSample=c(VectoSample,rep(k,subjectsPerCluster[k]))
+		}
+		VectoSample=sample(VectoSample,nSubjects,replace=FALSE)
+	}
+
 	
 	# Loop over subjects
 	for(i in 1:nSubjects){
-		if(i<=subjectsPerCluster[k]){
-			clusterData<-clusterSummary$clusterData[[k]]
+		if (includeCAR){
+			clusterData<-clusterSummary$clusterData[[VectoSample[i]]]
 		}else{
-			clusterData<-clusterSummary$clusterData[[k+1]]
-			k<-k+1
-			subjectsPerCluster[k]<-subjectsPerCluster[k]+subjectsPerCluster[k-1]
+			if(i<=subjectsPerCluster[k]){
+				clusterData<-clusterSummary$clusterData[[k]]
+			}else{
+				clusterData<-clusterSummary$clusterData[[k+1]]
+				k<-k+1
+				subjectsPerCluster[k]<-subjectsPerCluster[k]+subjectsPerCluster[k-1]
+			}
 		}
       
 		# Loop over covariates to generate the X data
@@ -106,7 +122,23 @@ generateSampleDataFile<-function(clusterSummary){
 		W<-NULL
 	}
 
-   	# Response Vector Y
+	# Spatial CAR term
+	if (includeCAR) {
+		tau=clusterSummary$TauCAR
+		.write_neigh(nSubjects)
+		M=.prec_Matrix()
+		El=eigen(M)
+		l=El$values
+		e=El$vectors
+		U=rnorm(nSubjects-1,0,1)
+		U=U*(1/sqrt(l[1:(nSubjects-1)]))
+		U=as.vector(e[,1:(nSubjects-1)]%*%matrix(U,ncol=1))
+		U=U/sqrt(tau)    
+	} else {
+		U=rep(0,nSubjects)
+	}
+
+	# Response Vector Y
 	Y<-rep(0,nSubjects)
 	outcomeType<-clusterSummary$outcomeType
 	if(nFixedEffects>0){
@@ -138,14 +170,17 @@ generateSampleDataFile<-function(clusterSummary){
 	# Loop over subjects
 	for(i in 1:nSubjects){
             
-		if(i<=subjectsPerCluster[k]){
-			theta<-clusterSummary$clusterData[[k]]$theta
-		}else{
-			theta<-clusterSummary$clusterData[[k+1]]$theta
-			k<-k+1
-			subjectsPerCluster[k]<-subjectsPerCluster[k]+subjectsPerCluster[k-1]
+		if (includeCAR){
+			theta<-clusterSummary$clusterData[[VectoSample[i]]]$theta
+		}else{        
+			if(i<=subjectsPerCluster[k]){
+				theta<-clusterSummary$clusterData[[k]]$theta
+			}else{
+				theta<-clusterSummary$clusterData[[k+1]]$theta
+				k<-k+1
+				subjectsPerCluster[k]<-subjectsPerCluster[k]+subjectsPerCluster[k-1]
+			}
 		}
-
 		mu<-theta
 		if(nFixedEffects>0){
 			if (outcomeType=='Categorical'){			
@@ -157,6 +192,7 @@ generateSampleDataFile<-function(clusterSummary){
 			}
 		} 
 		if(outcomeType=='Poisson'){
+			mu<-mu+U[i]
 			mu<-mu+log(offset[i])
 			Y[i]<-rpois(1,exp(mu))
 		}else if(outcomeType=='Bernoulli'){
@@ -184,6 +220,8 @@ generateSampleDataFile<-function(clusterSummary){
 	covNames<-paste('Variable',seq(1,nCovariates,1),sep="")
 	if(nFixedEffects>0){
 		fixEffNames<-paste('FixedEffects',seq(1,nFixedEffects,1),sep="")
+	} else {
+		fixEffNames=NULL
 	}
 	outData<-data.frame(cbind(matrix(Y),X))
 	colnames(outData) <- c("outcome",covNames)
@@ -217,6 +255,11 @@ generateSampleDataFile<-function(clusterSummary){
 		out$inputData <- outData
 		out$outcomeT <- "outcomeT"
 	}
+	if(clusterSummary$includeCAR){
+		out$uCAR=U
+		out$TauCAR=tau
+		out$Permutation=VectoSample
+	}
 	return(out)
 }
 
@@ -235,6 +278,8 @@ clusSummaryCategoricalDiscrete<-function(){list(
 	'missingDataProb'=0.001,
 	'nClusters'=3,
 	'clusterSizes'=c(200,300,400),
+	'includeCAR'=FALSE,
+	'TauCAR'=100,
 	'clusterData'=list(list('theta'=c(0,3,0.5),
 		'covariateProbs'=list(c(0.8,0.1,0.1),
 			c(0.8,0.1,0.1),
@@ -266,6 +311,8 @@ clusSummaryBernoulliDiscrete<-function(){
 	'missingDataProb'=0,
 	'nClusters'=5,
 	'clusterSizes'=c(200,200,200,200,200),
+	'includeCAR'=FALSE,
+	'TauCAR'=100,
 	'clusterData'=list(list('theta'=log(9),
 		'covariateProbs'=list(c(0.8,0.1,0.1),
 			c(0.8,0.1,0.1),
@@ -310,6 +357,8 @@ clusSummaryPoissonDiscrete<-function(){
 	'missingDataProb'=0.001,
 	'nClusters'=5,
 	'clusterSizes'=c(150,250,250,250,150),
+	'includeCAR'=FALSE,
+	'TauCAR'=100,
 	'clusterData'=list(list('theta'=log(10),
 		'covariateProbs'=list(c(0.8,0.2),
 			c(0.2,0.8),
@@ -354,6 +403,8 @@ clusSummaryNormalDiscrete<-function(){
    'missingDataProb'=0,
    'nClusters'=5,
    'clusterSizes'=c(500,700,300,600,900),
+   'includeCAR'=FALSE,
+   'TauCAR'=100,
    'clusterData'=list(list('theta'=10,
                            'covariateProbs'=list(c(0.8,0.1,0.1),
                                                  c(0.8,0.1,0.1),
@@ -397,6 +448,8 @@ clusSummaryPoissonNormal<-function(){
 	'missingDataProb'=0.001,
 	'nClusters'=3,
 	'clusterSizes'=c(500,600,400),
+	'includeCAR'=FALSE,
+	'TauCAR'=100,
 	'clusterData'=list(list('theta'=log(10),
 			'covariateMeans'=c(0,2),
 			'covariateCovariance'=matrix(c(0.5,0,0,3),nrow=2)),
@@ -408,6 +461,31 @@ clusSummaryPoissonNormal<-function(){
 			'covariateCovariance'=matrix(c(2,0.7,0.7,1),nrow=2))))
 }
 
+clusSummaryPoissonNormalSpatial<-function(){
+  list(
+    'outcomeType'='Poisson',
+    'covariateType'='Normal',
+    'nCovariates'=2,
+    'nFixedEffects'=2,
+    'fixedEffectsCoeffs'=c(-0.05,0.1),
+    'offsetLims'=c(0.9,1.1),
+    'missingDataProb'=0.001,
+    'nClusters'=3,
+    'clusterSizes'=c(50,60,40),
+    'includeCAR'=TRUE,
+    'TauCAR'=100,
+    'clusterData'=list(list('theta'=log(10),
+                            'covariateMeans'=c(0,2),
+                            'covariateCovariance'=matrix(c(0.5,0,0,3),nrow=2)),
+                       list('theta'=log(3),
+                            'covariateMeans'=c(3,2),
+                            'covariateCovariance'=matrix(c(1,0,0,1),nrow=2)),
+                       list('theta'=log(0.1),
+                            'covariateMeans'=c(10,-5),
+                            'covariateCovariance'=matrix(c(2,0.7,0.7,1),nrow=2))))
+}
+
+
 clusSummaryBinomialNormal<-function(){
 	list(
 	'outcomeType'='Binomial',
@@ -419,6 +497,8 @@ clusSummaryBinomialNormal<-function(){
 	'missingDataProb'=0.001,
 	'nClusters'=3,
 	'clusterSizes'=c(200,700,100),
+	'includeCAR'=FALSE,
+	'TauCAR'=100,
 	'clusterData'=list(list('theta'=log(10),
 			'covariateMeans'=c(0,2),
 			'covariateCovariance'=matrix(c(0.5,0,0,3),nrow=2)),
@@ -440,6 +520,8 @@ clusSummaryNormalNormal<-function(){
 	'missingDataProb'=0,
 	'nClusters'=3,
 	'clusterSizes'=c(300,500,400),
+	'includeCAR'=FALSE,
+	'TauCAR'=100,
 	'clusterData'=list(list('theta'=-5,
 			'covariateMeans'=c(0,2),
 			'covariateCovariance'=matrix(c(0.5,0,0,3),nrow=2)),
@@ -461,6 +543,8 @@ clusSummaryVarSelectBernoulliDiscrete<-function(){
 	'missingDataProb'=0,
 	'nClusters'=5,
 	'clusterSizes'=c(200,200,200,200,200),
+	'includeCAR'=FALSE,
+	'TauCAR'=100,
 	'clusterData'=list(list('theta'=log(1.0/9.0),
 	'covariateProbs'=list(c(0.9,0.1),
 			c(0.9,0.1),
@@ -530,6 +614,8 @@ clusSummaryBernoulliMixed<-function(){
 	'missingDataProb'=0,
 	'nClusters'=3,
 	'clusterSizes'=c(300,300,300),
+	'includeCAR'=FALSE,
+	'TauCAR'=100,
 	'clusterData'=list(list('theta'=log(10),
 		'covariateProbs'=list(c(0.8,0.1,0.1),
 			c(0.8,0.1,0.1),
@@ -562,6 +648,8 @@ clusSummaryBernoulliDiscreteSmall<-function(){
 	'missingDataProb'=0,
 	'nClusters'=5,
 	'clusterSizes'=c(100,100,100,100,100),
+	'includeCAR'=FALSE,
+	'TauCAR'=100,
 	'clusterData'=list(list('theta'=log(9),
 		'covariateProbs'=list(c(0.8,0.1,0.1),
 			c(0.8,0.1,0.1),
@@ -604,6 +692,8 @@ clusSummaryBernoulliNormal<-function(){
 	'missingDataProb'=0,
 	'nClusters'=5,
 	'clusterSizes'=c(100,100,100,100,100),
+	'includeCAR'=FALSE,
+	'TauCAR'=100,
 	'clusterData'=list(list('theta'=log(9),
 			'covariateMeans'=c(0,2),
 			'covariateCovariance'=matrix(c(0.5,0,0,3),nrow=2)),
@@ -622,3 +712,190 @@ clusSummaryBernoulliNormal<-function(){
 
 }
 
+
+
+
+
+
+############# Additional functions for spatial term
+
+
+#Function for mapping the spatial effect when data are generated
+mapforGeneratedData=function(u, del=NULL, palette='RGB', main=''){
+	#Fill the matrix with u
+	if (floor(sqrt(length(u)))==sqrt(length(u))){n_u=length(u)
+	} else {n_u=(floor(sqrt(length(u)))+1)^2}
+	u_compl=rep(NA, n_u)
+	u_compl[1:length(u)]=u
+	n=sqrt(n_u)
+	mat=matrix(NA,nrow=n,ncol=n)
+	for (i in 1:n){mat[i,]=u[((i-1)*n)+1:n]}
+	lagS=seq(0:n)
+	lagT=seq(0:n)
+
+	#Set plot parameters
+	if (palette=='BW'){paletteBleue<-colorRampPalette(c("Gray 92","Gray 16"))
+	}else{paletteBleue<-colorRampPalette(c("blue","cyan","yellow","orange","red"))}
+	if(is.null(del)){
+		deltot=seq(range(u,na.rm=TRUE)[1],range(u,na.rm=TRUE)[2],length.out=100)
+		deltot[1]<-deltot[1]-(deltot[2]-deltot[1])/100
+		deltot[length(deltot)]<-deltot[length(deltot)]+(deltot[length(deltot)]-deltot[length(deltot)-1])/100  
+		labs=c(seq(1,length(deltot),20),length(deltot))
+	}else{
+		labs=c(1)
+		deltot=c()
+		for(i in 1:(length(del)-1)){
+			deltot=c(deltot,seq(del[i],del[i+1],length.out=20))
+			if(deltot[length(deltot)]==del[i+1]){deltot=deltot[-length(deltot)]}
+			labs=c(labs,length(deltot)+1)
+		}
+		deltot=c(deltot,del[length(del)])
+	}
+	pardef=par(no.readonly=TRUE)
+	par(mar=c(4,4,4,8),tck=0.02,mgp=c(3,0.2,0),las=1)
+       
+	#Begin plot      
+	plot.new()
+	plot.window(xlim=range(lagS),ylim=range(lagT),xlab="oui", ylab="non")
+	for ( s in 1:length(mat[1,])){
+		for ( t in 1:length(mat[,1])){
+			rect(xleft=lagS[s],xright=lagS[s+1],ybottom=lagT[t],ytop=lagT[t+1],col=paletteBleue(length(deltot-1))[findInterval(mat[t,s],deltot,all.inside=TRUE)],border=NA)
+    		}
+	}
+                    
+	axis(1, at=lagS[seq(1,length(lagS),2)],labels=as.character(lagS[seq(1,length(lagS),2)]),tick=TRUE, pos=c(lagS[1],lagT[1]), padj=0, cex.axis=1)
+	axis(2, at=lagT[seq(1,length(lagT),2)],labels=as.character(lagT[seq(1,length(lagT),2)]),tick=TRUE,pos=c(lagS[1],lagT[1]), cex.axis=1)
+	axis(3, at=lagS[seq(1,length(lagS),2)],labels=FALSE,tick=TRUE, pos=c(lagT[length(lagT)],lagS[1]))
+	axis(4,at=lagT[seq(1,length(lagT),2)],labels=FALSE,tick=TRUE, pos=c(lagS[length(lagS)],lagT[length(lagT)]))
+	rect(xleft=range(lagS)[1],xright=range(lagS)[2],ybottom=range(lagT)[1],ytop=range(lagT)[2],border='black',lwd=1)        
+	mtext(main, side=3, line=0.5, cex=2)
+  
+	colors=.color.scale(paletteBleue(length(deltot)-1),deltot,name="", unit="",labels=labs)
+	.cs.draw(colors,border=NA,horiz=F, cex=1,digits=2, side=1, length=0.8, offset=0, pos=1, width = 0.06)
+	par(pardef)
+}
+
+
+########################################################################################################
+#function for producing text file with lattice neighbourhood structure that may be read 
+
+.write_neigh=function(nSubjects, file=NULL){
+  if (is.null(file)) {file='Neighbours.txt'}
+  if (floor(sqrt(nSubjects))==sqrt(nSubjects)){ n=sqrt(nSubjects)
+  }else{ n=floor(sqrt(nSubjects))+1 }
+  out=c(nSubjects)
+  for ( i in 1:nSubjects){
+    if (i <=n){
+      if (i==1) out=c(out, paste(i,2,2,1+n))
+      else if (i==n) out=c(out, paste(i,2,i-1,2*n))
+      else out=c(out, paste(i, 3, i-1,i+1,i+n))
+    } else if (i <=(nSubjects-n)){
+      if (floor((i-1)/n)==(i-1)/n) out=c(out, paste(i,3,i-n,i+1,i+n))
+      else if (floor(i/n)==i/n) out =c(out, paste(i,3,i-n,i-1,i+n))
+      else out =c(out, paste (i,4,i-n,i-1,i+1,i+n))
+    } else {
+      if (floor((i-1)/n)==(i-1)/n) out=c(out, paste(i,2,i-n,i+1))
+      else if (floor(i/n)==i/n) out =c(out, paste(i,2,i-n,i-1))
+      else if (i==nSubjects) {
+        if (floor((i-1)/n)==(i-1)/n) out=c(out,paste(i,1,i-n))
+        else out=c(out, paste(i,2,i-n,i-1))
+      } else {out =c(out, paste(i,3,i-n,i-1,i+1))}      
+    }
+  }
+  writeLines(out,file)
+}
+
+.prec_Matrix=function(file=NULL){
+  if (is.null(file)) file='Neighbours.txt'
+  con=file(file)
+  open(con)
+  nSubjects=as.numeric(readLines(con,n=1,warn=FALSE))
+  MAT=matrix(0,ncol=nSubjects,nrow=nSubjects)
+  for (i in 1:nSubjects){
+    tmp=as.numeric(strsplit(readLines(con,n=1,warn=FALSE),split=' ')[[1]])
+    MAT[i,i]=tmp[2]
+    MAT[i,tmp[2+1:tmp[2]]]=-1
+  }
+  close(con)
+  return(MAT)
+}
+
+.cs.draw <- function (color.scale, name = NULL, unit = NULL, length = 0.8,
+    width = 0.03, horiz = T, pos = 1.09, side = if (pos > 0.5) -1 else 1,
+    cex = 1, offset = 0, border = NULL, lty = NULL, lwd = par("lwd"),
+    xpd = T, digits = 4, roundfunc = zapsmall)
+{
+    nc <- length(color.scale$cols)
+    if (is.null(name))
+        name <- color.scale$name
+    if (is.null(unit))
+        unit <- color.scale$unit
+    if (horiz) {
+        xc <- (0.5 - offset) * par("usr")[1] + (0.5 + offset) *
+            par("usr")[2]
+        xd <- length * (par("usr")[2] - par("usr")[1])
+        x1 <- xc - xd/2
+        x2 <- xc + xd/2
+        x <- seq(x1, x2, , nc + 1)
+        ya <- par("usr")[4] - par("usr")[3]
+        yd <- width * (par("usr")[4] - par("usr")[3])
+        y1 <- par("usr")[3] + pos * ya
+        y2 <- y1 + side * yd
+        for (i in 1:nc) {
+            rect(x[i], y1, x[i + 1], y2, col = color.scale$cols[i],
+                border = border, lty = lty, lwd = lwd, xpd = xpd)
+            if (i %in% color.scale$labels)
+                text(x[i], y2, adj = c(0.5, -0.75 * side + 0.5),
+                  roundfunc(color.scale$breaks[i], digits = digits),
+                  xpd = xpd, cex = cex)
+        }
+        text(x[nc + 1], y2, adj = c(0.5, -0.75 * side + 0.5),
+            roundfunc(color.scale$breaks[nc + 1], digits = digits),
+            xpd = xpd, cex = cex)
+        text(xc, y1, adj = c(0.5, 0.75 * side + 0.5), name, xpd = xpd,
+            cex = cex)
+        text(x[nc + 1], (y1 + y2)/2, adj = c(-0.05, 0.5), unit,
+            xpd = xpd, cex = cex)
+    }
+    else {
+        yc <- (0.5 - offset) * par("usr")[3] + (0.5 + offset) *
+            par("usr")[4]
+        yd <- length * (par("usr")[4] - par("usr")[3])
+        y1 <- yc - yd/2
+        y2 <- yc + yd/2
+        y <- seq(y1, y2, , nc + 1)
+        xa <- par("usr")[2] - par("usr")[1]
+        xd <- width * (par("usr")[2] - par("usr")[1])
+        x1 <- par("usr")[1] + pos * xa
+        x2 <- x1 + side * xd
+        for (i in 1:nc) {
+            rect(x1, y[i], x2, y[i + 1], col = color.scale$cols[i],
+                border = border, lty = lty, lwd = lwd, xpd = xpd)
+            if (i %in% color.scale$labels)
+                text(x2, y[i], adj = c(-0.75 * side + 0.5, 0.5),
+                  roundfunc(color.scale$breaks[i], digits = digits),
+                  xpd = xpd, cex = cex)
+        }
+        text(x2, y[nc + 1], adj = c(-0.75 * side + 0.5, 0.5),
+            roundfunc(color.scale$breaks[nc + 1], digits = digits),
+            xpd = xpd, cex = cex)
+        text(x1, yc, adj = c(0.5, -0.75 * side + 0.5), name,
+            xpd = xpd, srt = 90, cex = cex)
+        text(x1, y[1], adj = c(-0.75 * side + 0.5, 1.5), unit,
+            xpd = xpd, cex = cex)
+    }
+} 
+
+#Function to create color.scale
+.color.scale=function(vectofcols, vectofbreaks, name="name", unit="", labels){
+  res=list()
+  res$cols=vectofcols
+  res$breaks=vectofbreaks
+  res$name=name
+  res$unit=unit
+  if (is.null(labels)){res$labels=seq(1,length(vectofbreaks),1)
+  }else{res$labels=labels}
+  return(res)
+}
+
+
